==> tiny-multiblock-lib-fabric-1.20.1-2.0-sources.jar: net/nikdo53/tinymultiblocklib/blockentities/AbstractMultiBlockEntity.java <==
package net.nikdo53.tinymultiblocklib.blockentities;

import net.minecraft.class_2338;
import net.minecraft.class_2487;
import net.minecraft.class_2512;
import net.minecraft.class_2586;
import net.minecraft.class_2591;
import net.minecraft.class_2596;
import net.minecraft.class_2602;
import net.minecraft.class_2622;
import net.minecraft.class_2680;
import net.nikdo53.tinymultiblocklib.block.AbstractMultiBlock;
import net.nikdo53.tinymultiblocklib.block.IMultiBlock;
import net.nikdo53.tinymultiblocklib.components.PreviewMode;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class AbstractMultiBlockEntity extends class_2586 implements IMultiBlockEntity{
    public class_2338 center;
    public boolean isPlaced;
    public PreviewMode previewMode = PreviewMode.PLACED;
    public List<class_2338> BLOCK_SHAPE_CACHE = new ArrayList<>();

    public AbstractMultiBlockEntity(class_2591<?> type, class_2338 pos, class_2680 state) {
        super(type, pos, state);
        this.center = this.method_11016();
        this.isPlaced = false;
    }

    @Override
    protected void method_11007(class_2487 tag) {
        super.method_11007(tag);
        tag.method_10566("center", class_2512.method_10692(this.center));
        tag.method_10556("placed", this.isPlaced);
    }

    @Override
    public class_2487 method_16887() {
        class_2487 tag = super.method_16887();
        method_11007(tag);
        return tag;
    }

    @Override
    public void method_11014(class_2487 tag) {
        super.method_11014(tag);
        this.center = class_2512.method_10691(tag.method_10562("center"));
        this.isPlaced = tag.method_10577("placed");
    }

    @Override
    public @Nullable class_2596<class_2602> method_38235() {
        return class_2622.method_38585(this);
    }

    @Override
    public class_2338 getCenter() {
        return this.center;
    }

    @Override
    public void setCenter(class_2338 pos) {
        this.center = pos;
    }

    @Override
    public boolean isPlaced() {
        return isPlaced;
    }

    @Override
    public void setPlaced(boolean placed) {
        this.isPlaced = placed;
    }

    @Override
    public List<class_2338> getFullBlockShapeCache() {
        return BLOCK_SHAPE_CACHE;
    }

    @Override
    public void setFullBlockShapeCache(List<class_2338> blockPosList) {
        BLOCK_SHAPE_CACHE = blockPosList;
    }

    @Override
    public void invalidateCaches() {
        BLOCK_SHAPE_CACHE = new ArrayList<>();
    }

    @Override
    public PreviewMode getPreviewMode() {
        return previewMode;
    }

    @Override
    public void setPreviewMode(PreviewMode mode) {
        this.previewMode = mode;
    }

    /**
     * Certain mods let you change the location of the multiblock (like Carry on)
     * <p>
     * That's a problem because the {@link #center} won't update. This should trick it into updating
     */
    @Override
    public void method_31664(class_2680 blockState) {
        if (IMultiBlock.isCenter(blockState)){
            setCenter(this.method_11016());
        }
        super.method_31664(blockState);
    }
}

==> tiny-multiblock-lib-fabric-1.20.1-2.2-sources.jar: net/nikdo53/tinymultiblocklib/blockentities/AbstractMultiBlockEntity.java <==
package net.nikdo53.tinymultiblocklib.blockentities;

import net.minecraft.class_2338;
import net.minecraft.class_2487;
import net.minecraft.class_2512;
import net.minecraft.class_2586;
import net.minecraft.class_2591;
import net.minecraft.class_2596;
import net.minecraft.class_2602;
import net.minecraft.class_2622;
import net.minecraft.class_2680;
import net.nikdo53.tinymultiblocklib.block.AbstractMultiBlock;
import net.nikdo53.tinymultiblocklib.block.IMultiBlock;
import net.nikdo53.tinymultiblocklib.components.PreviewMode;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class AbstractMultiBlockEntity extends class_2586 implements IMultiBlockEntity{
    private class_2338 offset;
    private boolean isPlaced;
    private PreviewMode previewMode = PreviewMode.PLACED;
    private List<class_2338> BLOCK_SHAPE_CACHE = new ArrayList<>();

    public AbstractMultiBlockEntity(class_2591<?> type, class_2338 pos, class_2680 state) {
        super(type, pos, state);
        this.offset = new class_2338(0,0,0);
        this.isPlaced = false;
    }

    @Override
    protected void method_11007(class_2487 tag) {
        super.method_11007(tag);
        tag.method_10566("offset", class_2512.method_10692(this.offset));
        tag.method_10556("placed", this.isPlaced);
    }

    @Override
    public class_2487 method_16887() {
        class_2487 tag = super.method_16887();
        method_11007(tag);
        return tag;
    }

    @Override
    public void method_11014(class_2487 tag) {
        super.method_11014(tag);
        this.offset = class_2512.method_10691(tag.method_10562("offset"));
        this.isPlaced = tag.method_10577("placed");

        if (tag.method_10545("center")) // For maintaining compatibility with TMBL < 2.1
            setCenter(class_2512.method_10691(tag.method_10562("center")));
    }

    @Override
    public @Nullable class_2596<class_2602> method_38235() {
        return class_2622.method_38585(this);
    }

    @Override
    public class_2338 getOffset() {
        return offset;
    }

    @Override
    public void setOffset(class_2338 offset) {
       this.offset = offset;
    }

    @Override
    public boolean isPlaced() {
        return isPlaced;
    }

    @Override
    public void setPlaced(boolean placed) {
        this.isPlaced = placed;
    }

    @Override
    public List<class_2338> getFullBlockShapeCache() {
        return BLOCK_SHAPE_CACHE;
    }

    @Override
    public void setFullBlockShapeCache(List<class_2338> blockPosList) {
        BLOCK_SHAPE_CACHE = blockPosList;
    }

    @Override
    public void invalidateCaches() {
        BLOCK_SHAPE_CACHE = new ArrayList<>();
    }

    @Override
    public PreviewMode getPreviewMode() {
        return previewMode;
    }

    @Override
    public void setPreviewMode(PreviewMode mode) {
        this.previewMode = mode;
    }
}
